@@ -22,6 +22,7 @@
 #include <ggv-control.h>
 #include <gsdefaults.h>
 #include <ggvutils.h>
+#include <ggv-document.h>
 
 struct _GgvControlPrivate {
 	GgvPostScriptView *ps_view;
@@ -165,6 +166,95 @@ sync_command_state(GgvControl *control)
 	sync_zoom_level_items(control);
 }
 
+static void
+_set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+	Bonobo_UIContainer container;
+	gint i;
+
+	container = bonobo_ui_component_get_container (uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	bonobo_ui_component_freeze(uic, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomIn",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomOut",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToDefault",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFit",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFitWidth",
+								 "sensitive", prop_val, NULL);
+	for(i = 0; zoom_level_items[i].path != NULL; i++) {
+		bonobo_ui_component_set_prop(uic, zoom_level_items[i].path,
+									 "sensitive", prop_val, NULL);
+	}
+	bonobo_ui_component_thaw(uic, NULL);
+}
+
+static void
+set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	bonobo_ui_component_set_prop(control->priv->uic,
+								 "/Sidebar/Zoom Control Placeholder/ZoomControl",
+								 "sensitive", sens?"1":"0", NULL);
+	_set_zoom_items_sensitivity(control, control->priv->uic, sens);
+	if(control->priv->popup_uic != NULL)
+		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
+}
+
+static void
+set_auto_fit_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+	Bonobo_UIContainer container;
+
+	container = bonobo_ui_component_get_container (control->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	bonobo_ui_component_freeze(control->priv->uic, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitNone",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitWidth",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitPage",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(control->priv->uic, NULL);	
+}
+
+static gboolean
+is_auto_fit_set(GgvControl *control)
+{
+	GgvPostScriptView *ps_view;
+	GtkWidget *gs;
+
+	ps_view = control->priv->ps_view;
+	gs = ggv_postscript_view_get_widget(ps_view);
+
+	switch(gtk_gs_get_zoom_mode(GTK_GS(gs))) {
+	case GTK_GS_ZOOM_ABSOLUTE:
+		return FALSE;
+	case GTK_GS_ZOOM_FIT_WIDTH:
+	case GTK_GS_ZOOM_FIT_PAGE:
+		return TRUE;
+	default:
+		return FALSE;
+	}
+
+}
+
+static void
+set_command_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	set_auto_fit_items_sensitivity(control, sens);
+	sens = sens & !is_auto_fit_set(control);
+	set_zoom_items_sensitivity(control, sens);
+}
+
 static void
 zoomable_set_frame_cb (BonoboZoomable *zoomable, GgvControl *control)
 {
@@ -289,6 +379,20 @@ scrollbar_button_release_event(GtkWidget *widget, GdkEventButton *event,
 	return FALSE;
 }
 
+static void
+sw_size_allocate(GtkWidget *widget, GtkAllocation *alloc, gpointer data)
+{
+	GtkGS *view = GTK_GS(data);
+	guint border = gtk_container_get_border_width(GTK_CONTAINER(widget));
+	gint w, h;
+
+	w = alloc->width - 2*border;
+	h = alloc->height - 2*border;
+	w = MAX(w, 0);
+	h = MAX(h, 0);
+	gtk_gs_set_available_size(view, w, h);
+}
+
 static void
 listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
 					  Bonobo_UIComponent_EventType type, const char *state,
@@ -343,12 +447,18 @@ listener_AutoFitMode_cb(BonoboUIComponent *uic, const char *path,
 	control = GGV_CONTROL(user_data);
 	gs = GTK_GS(ggv_postscript_view_get_widget(control->priv->ps_view));
 
-	if(!strcmp(path, "AutoFitNone"))
+	if(!strcmp(path, "AutoFitNone")) {
 		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_ABSOLUTE);
-	if(!strcmp(path, "AutoFitWidth"))
+		set_zoom_items_sensitivity(control, TRUE);
+	}
+	if(!strcmp(path, "AutoFitWidth")) {
 		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_WIDTH);
-	if(!strcmp(path, "AutoFitPage"))
+		set_zoom_items_sensitivity(control, FALSE);
+	}
+	if(!strcmp(path, "AutoFitPage")) {
 		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_PAGE);
+		set_zoom_items_sensitivity(control, FALSE);
+	}
 
 	gtk_widget_unref(GTK_WIDGET(gs));
 }
@@ -365,6 +475,14 @@ ggv_control_button_press_event(GtkWidget *widget, GdkEventButton *event,
 	return FALSE;
 }
 
+static gboolean
+ggv_control_popup_menu(GtkWidget *widget, gpointer data)
+{
+	GgvControl *control = GGV_CONTROL(data);
+
+	return bonobo_control_do_popup(BONOBO_CONTROL(control), 3, GDK_CURRENT_TIME);
+}
+
 static gboolean
 ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 							gpointer data)
@@ -374,6 +492,7 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	GgvControl *control = GGV_CONTROL(data);
 	GtkGSOrientation orientation = gtk_gs_get_orientation(gs);
 	GgvPostScriptView *ps_view = control->priv->ps_view;
+	gfloat val;
 
 	/* ugh. the possibilities! */
 	switch (key) {
@@ -383,21 +502,33 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		case GTK_GS_ORIENTATION_LANDSCAPE:
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		case GTK_GS_ORIENTATION_SEASCAPE:
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		case GTK_GS_ORIENTATION_UPSIDEDOWN:
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		default:
 			break;
@@ -410,52 +541,69 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		case GTK_GS_ORIENTATION_LANDSCAPE:
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		case GTK_GS_ORIENTATION_SEASCAPE:
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		case GTK_GS_ORIENTATION_UPSIDEDOWN:
 			if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, TRUE)) {
 				ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 			}
+			else
+				gtk_widget_queue_draw
+					(GTK_SCROLLED_WINDOW(control->priv->root)->vscrollbar);
 			break;
 		default:
 			break;
 		}
 		break;
 	case GDK_Left:
-		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, TRUE) &&
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE) &&
 		   ggv_postscript_view_get_page_flip(ps_view))
 			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Right:
-		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, TRUE) &&
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE) &&
 		   ggv_postscript_view_get_page_flip(ps_view))
 			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Up:
-		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, TRUE) &&
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE) &&
 		   ggv_postscript_view_get_page_flip(ps_view))
 			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Down:
-		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, TRUE) &&
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, FALSE) &&
 		   ggv_postscript_view_get_page_flip(ps_view))
 			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Page_Up:
-		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
+	case GDK_KP_Page_Up:
+		gtk_adjustment_set_value(gs->vadj, gs->vadj->value - gs->vadj->page_increment);
 		break;
 	case GDK_Page_Down:
-		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
-		break;
+	case GDK_KP_Page_Down:
+		val = gs->vadj->value + gs->vadj->page_increment;
+		if (val > gs->vadj->upper - gs->vadj->page_size)
+			val = gs->vadj->upper - gs->vadj->page_size;
+		gtk_adjustment_set_value(gs->vadj, val);
+			break;
 	case GDK_plus:
 		g_signal_emit_by_name(G_OBJECT (control->priv->zoomable),
 							  "zoom_in");
@@ -548,70 +696,6 @@ verb_ZoomToFitWidth_cb (BonoboUIComponent *uic, gpointer user_data,
 
 }
 
-static void
-_set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolean sens)
-{
-	gchar *prop_val = sens?"1":"0";
-	Bonobo_UIContainer container;
-	gint i;
-
-	container = bonobo_ui_component_get_container (uic);
-	if (container == CORBA_OBJECT_NIL)
-		return;
-
-	bonobo_ui_component_freeze(uic, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomIn",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomOut",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomToDefault",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFit",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFitWidth",
-								 "sensitive", prop_val, NULL);
-	for(i = 0; zoom_level_items[i].path != NULL; i++) {
-		bonobo_ui_component_set_prop(uic, zoom_level_items[i].path,
-									 "sensitive", prop_val, NULL);
-	}
-	bonobo_ui_component_thaw(uic, NULL);
-}
-
-static void
-set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
-{
-	_set_zoom_items_sensitivity(control, control->priv->uic, sens);
-	if(control->priv->popup_uic != NULL)
-		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
-}
-
-static void
-set_auto_fit_items_sensitivity(GgvControl *control, gboolean sens)
-{
-	gchar *prop_val = sens?"1":"0";
-	Bonobo_UIContainer container;
-
-	container = bonobo_ui_component_get_container (control->priv->uic);
-	if (container == CORBA_OBJECT_NIL)
-		return;
-
-	bonobo_ui_component_freeze(control->priv->uic, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitNone",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitWidth",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitPage",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_thaw(control->priv->uic, NULL);	
-}
-
-static void
-set_command_items_sensitivity(GgvControl *control, gboolean sens)
-{
-	set_auto_fit_items_sensitivity(control, sens);
-	set_zoom_items_sensitivity(control, sens);
-}
-
 static BonoboUIVerb ggv_control_verbs[] = {
 		BONOBO_UI_VERB ("ZoomIn",         verb_ZoomIn_cb),
 		BONOBO_UI_VERB ("ZoomOut",        verb_ZoomOut_cb),
@@ -660,7 +744,7 @@ ggv_control_create_ui (GgvControl *control)
 	if(pixbuf != NULL) {
 		pixbuf_xml = bonobo_ui_util_pixbuf_to_xml(pixbuf);
 		bonobo_ui_component_set_prop(control->priv->uic,
-									 "/Toolbar/GgvItems/GgvZoomItems/ZoomToFitWidth",
+									 "/Toolbar/Viewer Items Placeholder/Zoom Items Placeholder/ZoomToFitWidth",
 									 "pixname", pixbuf_xml,
 									 NULL);
 		g_free(pixbuf_xml);
@@ -694,7 +778,7 @@ ggv_control_create_ui (GgvControl *control)
 									 "AutoFitPage",
 									 listener_AutoFitMode_cb, control);
 
-	set_command_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
+	set_command_items_sensitivity(control, ggv_document_get_page_count(GGV_DOCUMENT(control->priv->ps_view)) > 0);
 
 	sync_command_state(control);
 
@@ -862,6 +946,8 @@ ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 					 G_CALLBACK(ggv_control_key_press_event), control);
 	g_signal_connect(G_OBJECT(view), "button_press_event",
 					 G_CALLBACK(ggv_control_button_press_event), control);
+	g_signal_connect(G_OBJECT(view), "popup_menu",
+					 G_CALLBACK(ggv_control_popup_menu), control);
 
 	control->priv->root = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(control->priv->root);
@@ -885,6 +971,10 @@ ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 					 "button_release_event",
 					 G_CALLBACK(scrollbar_button_release_event),
 					 view);
+	g_signal_connect(G_OBJECT(GTK_SCROLLED_WINDOW(control->priv->root)),
+					 "size_allocate",
+					 G_CALLBACK(sw_size_allocate),
+					 view);
 	gtk_container_add(GTK_CONTAINER(control->priv->root), view);
 	
 	bonobo_control_construct (BONOBO_CONTROL (control), control->priv->root);
