@@ -20,18 +20,21 @@
 
 #include <ggv-sidebar.h>
 #include <gtkchecklist.h>
-#include <gsdefaults.h>
+#include <ggv-document.h>
+
 #include <ggvutils.h>
 #include <ggv-prefs.h>
 
 struct _GgvSidebarPrivate {
-	GgvPostScriptView *ps_view;
+	GgvDocument *document;
 
 	GtkWidget *root;
 
 	GtkWidget *checklist;
 	GtkWidget *toggle_all, *toggle_even, *toggle_odd, *clear_all;
 	GtkWidget *coordinates;
+
+	GtkTooltips *toggle_tips;
 };
 
 struct _GgvSidebarClassPrivate {
@@ -40,6 +43,13 @@ struct _GgvSidebarClassPrivate {
 
 static BonoboControlClass *ggv_sidebar_parent_class;
 
+enum {
+	SET_PAGE_SIGNAL,
+	LAST_SIGNAL
+};
+
+static guint ggv_sidebar_signals [LAST_SIGNAL];
+
 static void
 ggv_sidebar_destroy (BonoboObject *object)
 {
@@ -64,6 +74,9 @@ ggv_sidebar_finalize (GObject *object)
 
 	sidebar = GGV_SIDEBAR (object);
 
+	if(sidebar->priv->toggle_tips)
+		g_object_unref (G_OBJECT(sidebar->priv->toggle_tips));
+
 	g_free (sidebar->priv);
 
 	G_OBJECT_CLASS (ggv_sidebar_parent_class)->finalize (object);
@@ -81,6 +94,15 @@ ggv_sidebar_class_init (GgvSidebarClass *klass)
 	object_class->finalize = ggv_sidebar_finalize;
 
 	klass->priv = g_new0(GgvSidebarClassPrivate, 1);
+
+	ggv_sidebar_signals[SET_PAGE_SIGNAL] = g_signal_new (
+		"set_page",
+		G_TYPE_FROM_CLASS (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GgvSidebarClass, set_page),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__INT,
+		G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
 static void
@@ -184,10 +206,15 @@ page_list_selection_changed(GtkTreeSelection *sel,
 		path = gtk_tree_model_get_path(model,
 									   &iter);
 		path_str = gtk_tree_path_to_string(path);
-		page = atoi(path_str);
+		page = atoi(path_str); /* FIXME atoi is locale dependant */
 		g_free(path_str);
 		if(page >= 0)
-			ggv_postscript_view_goto_page(sidebar->priv->ps_view, page);
+			g_signal_emit (G_OBJECT (sidebar),
+						   ggv_sidebar_signals[SET_PAGE_SIGNAL],
+						   0, page);
+		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(sidebar->priv->checklist),
+									 path, NULL, FALSE, 0.0, 0.0);
+		gtk_tree_path_free(path);
 	}
 }
 
@@ -213,8 +240,8 @@ ggv_sidebar_create_page_list(GgvSidebar *sidebar)
 
 	gtk_check_list_clear(GTK_CHECK_LIST(sidebar->priv->checklist));
 
-	page_count = ggv_postscript_view_get_page_count(sidebar->priv->ps_view);
-	page_names = ggv_postscript_view_get_page_names(sidebar->priv->ps_view);
+	page_count = ggv_document_get_page_count(sidebar->priv->document);
+	page_names = ggv_document_get_page_names(sidebar->priv->document);
 	if(page_count <= 0)
 		return;
 	if(page_names == NULL)
@@ -245,9 +272,12 @@ ggv_sidebar_page_changed(GgvSidebar *sidebar, gint page)
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(sidebar->priv->checklist));
 	g_snprintf(path_str, 15, "%d", page);
 	path = gtk_tree_path_new_from_string(path_str);
-	if(!gtk_tree_selection_path_is_selected(sel, path))
+	if(!gtk_tree_selection_path_is_selected(sel, path)) {
 		gtk_tree_selection_select_path(sel, path);
-	gtk_tree_path_free(path);	
+	}
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(sidebar->priv->checklist),
+								 path, NULL, FALSE, 0.0, 0.0);
+	gtk_tree_path_free(path);
 }
 
 gint *
@@ -259,26 +289,32 @@ ggv_sidebar_get_active_list(GgvSidebar *sidebar)
 BONOBO_TYPE_FUNC (GgvSidebar, BONOBO_TYPE_CONTROL, ggv_sidebar);
 
 GgvSidebar *
-ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
+ggv_sidebar_construct (GgvSidebar *sidebar, GgvDocument *doc)
 {
 	GtkWidget *hbox, *image, *sw;
 	GtkTreeSelection *sel;
 
-	g_return_val_if_fail (ps_view != NULL, NULL);
+	g_return_val_if_fail (doc != NULL, NULL);
 	g_return_val_if_fail (sidebar != NULL, NULL);
-	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (ps_view), NULL);
+	g_return_val_if_fail (GGV_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (GGV_IS_SIDEBAR (sidebar), NULL);
 
-	sidebar->priv->ps_view = ps_view;
+	sidebar->priv->document = doc;
 
 	/* with a sidebar on the left */
 	sidebar->priv->root = gtk_vbox_new(FALSE, 0);
 
-	sidebar->priv->coordinates = gtk_label_new(" ");
+	sidebar->priv->toggle_tips = gtk_tooltips_new();
+	g_object_ref(G_OBJECT(sidebar->priv->toggle_tips));
+	gtk_object_sink(GTK_OBJECT(sidebar->priv->toggle_tips));
+
+	sidebar->priv->coordinates = gtk_entry_new();
+	gtk_widget_set_size_request(sidebar->priv->coordinates, 0, -1);
+	gtk_editable_set_editable(GTK_EDITABLE(sidebar->priv->coordinates), FALSE);
 	gtk_widget_show(sidebar->priv->coordinates);
 	gtk_box_pack_start(GTK_BOX(sidebar->priv->root),
 					   sidebar->priv->coordinates,
-					   FALSE, TRUE, 2);
+					   FALSE, FALSE, 2);
 
 	hbox = gtk_hbox_new(TRUE, 2);
 	gtk_widget_show(hbox);
@@ -288,6 +324,11 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_widget_show(image);
 	sidebar->priv->toggle_all = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_all), GTK_RELIEF_NONE);
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_all,
+						 _("Toggle marked state of all pages"),
+						 _("Toggle marked state of all pages: previously "
+						   "marked pages will be unmarked and unmarked ones "
+						   "will become marked."));
 	gtk_widget_show(sidebar->priv->toggle_all);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_all, FALSE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(sidebar->priv->toggle_all), image);
@@ -296,6 +337,11 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleodd.xpm");
 	gtk_widget_show(image);
 	sidebar->priv->toggle_odd = gtk_button_new();
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_odd,
+						 _("Toggle marked state of odd pages"),
+						 _("Toggle marked state of odd pages: previously "
+						   "marked odd pages will be unmarked and unmarked "
+						   "ones will become marked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_odd), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->toggle_odd);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_odd, FALSE, TRUE, 0);
@@ -305,6 +351,11 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/toggleeven.xpm");
 	gtk_widget_show(image);
 	sidebar->priv->toggle_even = gtk_button_new();
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_even,
+						 _("Toggle marked state of even pages"),
+						 _("Toggle marked state of even pages: previously "
+						   "marked even pages will be unmarked and unmarked "
+						   "ones will become marked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_even), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->toggle_even);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_even, FALSE, TRUE, 0);
@@ -314,6 +365,10 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/clearall.xpm");
 	gtk_widget_show(image);
 	sidebar->priv->clear_all = gtk_button_new();
+	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->clear_all,
+						 _("Clear marked state of all pages"),
+						 _("Clear marked state of all pages: all pages will "
+						   "be unmarked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->clear_all), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->clear_all);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->clear_all, FALSE, TRUE, 0);
@@ -345,16 +400,16 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 }
 
 GgvSidebar *
-ggv_sidebar_new (GgvPostScriptView *ps_view)
+ggv_sidebar_new (GgvDocument *doc)
 {
 	GgvSidebar *sidebar;
 	
-	g_return_val_if_fail (ps_view != NULL, NULL);
-	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (ps_view), NULL);
+	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (GGV_IS_DOCUMENT (doc), NULL);
 
 	sidebar = g_object_new(GGV_SIDEBAR_TYPE, NULL);
 
-	return ggv_sidebar_construct (sidebar, ps_view);
+	return ggv_sidebar_construct (sidebar, doc);
 }
 
 void
@@ -365,7 +420,13 @@ ggv_sidebar_update_coordinates(GgvSidebar *sidebar, gfloat xcoord, gfloat ycoord
 		clabel = g_strdup_printf("%.2f , %.2f",
 								 xcoord*ggv_unit_factors[ggv_unit_index],
 								 ycoord*ggv_unit_factors[ggv_unit_index]);
-		gtk_label_set_text(GTK_LABEL(sidebar->priv->coordinates), clabel);
+		gtk_entry_set_text(GTK_ENTRY(sidebar->priv->coordinates), clabel);
 		g_free(clabel);
 	}
 }
+
+GtkWidget *
+ggv_sidebar_get_checklist(GgvSidebar *sidebar)
+{
+	return sidebar->priv->checklist;
+}
