@@ -5,24 +5,29 @@
 #include <ctype.h>
 #include <sys/stat.h>
 
+#include <libgnomevfs/gnome-vfs-utils.h>
+
 #include "ggvutils.h"
 
 GtkGSPaperSize ggv_paper_sizes[] = {
-        {"BBox",         0,  0},
-        {"Letter",       612,  792,},
-        {"Tabloid",      792, 1224,},
-        {"Ledger",       1224,  792,},
-        {"Legal",        612, 1008,},
-        {"Statement",	 396,  612,},
-        {"Executive",	 540,  720,},
-        {"A3",		 842, 1190,},
-        {"A4",		 595,  842,},
-        {"A5",		 420,  595,},
-        {"B4",		 729, 1032,},
-        {"B5",		 516,  729,},
-        {"Folio",        612,  936,},
-        {"Quarto",       610,  780,},
-        {"10x14",        720, 1008,},
+        {N_("BBox"),         0,  0},
+        {N_("Letter"),       612,  792,},
+        {N_("Tabloid"),      792, 1224,},
+        {N_("Ledger"),       1224,  792,},
+        {N_("Legal"),        612, 1008,},
+        {N_("Statement"),	 396,  612,},
+        {N_("Executive"),	 540,  720,},
+        {N_("A0"),           2380, 3368,},
+        {N_("A1"),           1684, 2380,},
+        {N_("A2"),           1190, 1684,},
+        {N_("A3"),		 842, 1190,},
+        {N_("A4"),		 595,  842,},
+        {N_("A5"),		 420,  595,},
+        {N_("B4"),		 729, 1032,},
+        {N_("B5"),		 516,  729,},
+        {N_("Folio"),        612,  936,},
+        {N_("Quarto"),       610,  780,},
+        {N_("10x14"),        720, 1008,},
         {NULL,		   0,    0}
 };
 
@@ -56,20 +61,27 @@ const gint ggv_max_unit_labels =
     (sizeof(ggv_unit_labels) / sizeof(gchar *)) - 2;
 
 gfloat ggv_zoom_levels[] = {
-	1.0 / 10.0, 1.0 / 9.0, 1.0 / 8.0, 1.0 / 7.0, 1.0 / 6.0,
-	1.0 / 5.0, 1.0 / 4.0, 1.0 / 3.0, 1.0 / 2.0, 1.0, 2.0,
-	3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0
+	1.0 / 6.0, 1.0 / 5.0, 1.0 / 4.0, 1.0 / 3.0, 1.0 / 2.0, 3.0 / 4.0, 1.0,
+        3.0 / 2.0, 2.0,	3.0, 4.0, 5.0, 6.0
 };
 
 const gchar *ggv_zoom_level_names[] = {
-	"1:10", "1:9", "1:8", "1:7", "1:6", "1:5", "1:4", "1:3",
-	"1:2", "1:1", "2:1", "3:1", "4:1", "5:1", "6:1", "7:1",
-	"8:1", "9:1", "10:1"
+	"1:6", "1:5", "1:4", "1:3",
+	"1:2", "3:4", "1:1", "3:2",
+        "2:1", "3:1", "4:1", "5:1",
+        "6:1",
 };
 
 const gint ggv_max_zoom_levels =
     (sizeof (ggv_zoom_levels) / sizeof (gfloat)) - 1;
 
+const gchar *ggv_auto_fit_modes[] = {
+        N_("None"), N_("Fit to page width"), N_("Fit to page size")
+};
+
+const gint ggv_max_auto_fit_modes =
+    (sizeof (ggv_auto_fit_modes) / sizeof (gchar *)) - 1;
+
 gint
 ggv_zoom_index_from_float (gfloat zoom_level)
 {
@@ -167,52 +179,28 @@ ggv_get_index_of_string(gchar *string, gchar **strings)
 gchar *
 ggv_quote_filename (const gchar *str)
 {
-        gchar c, *s, *t, *buf;
+        return g_shell_quote(str);
+}
 
-        s = (gchar *) str;
-        t = buf = g_new (gchar, 2 * strlen (str) + 1);
-
-        while ((c = *s++) != '\0') {
-                switch (c) {
-                case ' ':
-                case '\t':
-                case '\n': /* white space */
-                case '\'':
-                case '"':
-                case '\\': /* quoting chars */
-                case '|':
-                case '&':
-                case ';':  /* metacharacters */
-                case '(':
-                case ')':
-                case '<':
-                case '>':
-                case '!':
-                case '{':
-                case '}':  /* reserved characters */
-                case '*':
-                case '[':
-                case '?':
-                case ']':  /* globbing chars */
-                case '^':
-                case '$':
-                case '`':  /* expansion characters */
-                case '%':
-                        *t++ = '\\';
-                        *t++ = c;
-                        break;
-                case '#':  /* comment char */
-                        if (s == str)
-                                *t++ = '\\';
-                        *t++ = c;
-                        break;
-                default:
-                        *t++ = c;
-                        break;
-                }
+/* escapes filename to form a proper URI: works conservatively - anything
+   except [a-zA-Z0-9_] will be escaped with a %XX escape sequence where
+   XX is the hex value of the char. */
+gchar *
+ggv_filename_to_uri(const gchar *fname)
+{
+        gchar *full_path, *ret_val;
+        
+        if(*fname != '/') {
+                gchar *cwd;
+                /* relative file name - we will have to absolutize it */
+                cwd = g_get_current_dir();
+                full_path = g_strconcat(cwd, "/", fname, NULL);
         }
-        *t++ = '\0';
-        return buf;
+        else full_path = NULL;
+        ret_val = gnome_vfs_get_uri_from_local_path(full_path?full_path:fname);
+        if(full_path)
+                g_free(full_path);
+        return ret_val;
 }
 
 /* If file exists and is a regular file then return its length, else -1 */
